/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.micode.notes.model;

import android.content.ContentProviderOperation;//更新、插入、删除数据
import android.content.ContentProviderResult;
import android.content.ContentUris;
import android.content.ContentValues;//存储基本数据类型数据
import android.content.Context;
import android.content.OperationApplicationException;//操作数据容错
import android.net.Uri;
import android.os.RemoteException;
import android.util.Log;

import net.micode.notes.data.Notes;
import net.micode.notes.data.Notes.CallNote;
import net.micode.notes.data.Notes.DataColumns;//小米便签数据处理操作（数据栏）
import net.micode.notes.data.Notes.NoteColumns;
import net.micode.notes.data.Notes.TextNote;//小米便签数据处理操作（文本操作）

import java.util.ArrayList;

public class Note {
    // 这个类是用来刻画单个Note的
    // private ContentValues mNoteDiffValues;
    ContentValues mNoteDiffValues;
    // ContentValues是用于给其他应用调用小米便签的内容的共享数据
    private NoteData mNoteData;
    private static final String TAG = "Note";// 软件的名称

    /**
     * Create a new note id for adding a new note to databases
     */
    public static synchronized long getNewNoteId(Context context, long folderId) {
        // Create a new note in the database
        ContentValues values = new ContentValues();
        // Create a new note in the database
        long createdTime = System.currentTimeMillis();
        // 读取当前系统时间
        values.put(NoteColumns.CREATED_DATE, createdTime);
        // 将创建时间和修改时间都更改为当前系统时间
        values.put(NoteColumns.MODIFIED_DATE, createdTime);
        // 更改时间
        values.put(NoteColumns.TYPE, Notes.TYPE_NOTE);
        // 便签类型
        values.put(NoteColumns.LOCAL_MODIFIED, 1);
        values.put(NoteColumns.PARENT_ID, folderId);// 将数据写入数据库表格
        Uri uri = context.getContentResolver().insert(Notes.CONTENT_NOTE_URI, values);
        // ContentResolver()主要是实现外部应用对ContentProvider中的数据
        // 进行添加、删除、修改和查询操作
        long noteId = 0;
        try {
            noteId = Long.valueOf(uri.getPathSegments().get(1));
            // 获取便签的id
        } catch (NumberFormatException e) {
            Log.e(TAG, "Get note id error :" + e.toString());
            // 获取id错误
            noteId = 0;
        } // try-catch异常处理
        if (noteId == -1) {
            throw new IllegalStateException("Wrong note id:" + noteId);
            // 非法状态时返回出错便签编号
        }
        return noteId;
    }

    public Note() {
        mNoteDiffValues = new ContentValues();
        mNoteData = new NoteData();
    }// 定义两个变量用来存储便签的数据，一个是存储便签属性、一个是存储便签内容

    public void setNoteValue(String key, String value) {
        mNoteDiffValues.put(key, value);
        mNoteDiffValues.put(NoteColumns.LOCAL_MODIFIED, 1);
        mNoteDiffValues.put(NoteColumns.MODIFIED_DATE, System.currentTimeMillis());
    }// 设置数据库表格的标签属性数据

    public void setTextData(String key, String value) {
        mNoteData.setTextData(key, value);
    }// 设置数据库表格的标签文本内容的数据

    public void setTextDataId(long id) {
        mNoteData.setTextDataId(id);
    }// 设置文本数据的ID

    public long getTextDataId() {
        return mNoteData.mTextDataId;
    }// 得到文本数据的ID

    public void setCallDataId(long id) {
        mNoteData.setCallDataId(id);
    }// 设置电话号码数据的ID

    public void setCallData(String key, String value) {
        mNoteData.setCallData(key, value);
    }// 得到电话号码数据的ID

    public boolean isLocalModified() {
        return mNoteDiffValues.size() > 0 || mNoteData.isLocalModified();
    }// 判断是否是本地修改

    public boolean syncNote(Context context, long noteId) {
        if (noteId <= 0) {
            throw new IllegalArgumentException("Wrong note id:" + noteId);
        }

        if (!isLocalModified()) {
            return true;
        }

        /**
         * In theory, once data changed, the note should be updated on
         * {@link NoteColumns#LOCAL_MODIFIED} and
         * {@link NoteColumns#MODIFIED_DATE}. For data safety, though update note fails,
         * we also update the
         * note data info
         */
        if (context.getContentResolver().update(
                ContentUris.withAppendedId(Notes.CONTENT_NOTE_URI, noteId), mNoteDiffValues, null,
                null) == 0) {
            Log.e(TAG, "Update note error, should not happen");
            // Do not return, fall through
        }
        mNoteDiffValues.clear();
        // 初始化便签特征值
        if (mNoteData.isLocalModified()
                && (mNoteData.pushIntoContentResolver(context, noteId) == null)) {
            // 如果向内容接收者推送上下文失败，则返回错误
            return false;
        }

        return true;
    }// 判断数据是否同步

    private class NoteData {// 定义一个基本的便签内容的数据类，主要包含文本数据和电话号码数据
        private long mTextDataId;

        private ContentValues mTextDataValues;// 文本数据

        private long mCallDataId;

        private ContentValues mCallDataValues;// 电话号码数据

        private static final String TAG = "NoteData";

        public NoteData() {
            // 定义一个基本的便签内容的数据类，主要包含文本数据和电话号码数据
            mTextDataValues = new ContentValues();
            mCallDataValues = new ContentValues();
            mTextDataId = 0;
            mCallDataId = 0;
        }

        // 下面是上述几个函数的具体实现
        boolean isLocalModified() {
            return mTextDataValues.size() > 0 || mCallDataValues.size() > 0;
        }

        void setTextDataId(long id) {
            if (id <= 0) {
                throw new IllegalArgumentException("Text data id should larger than 0");
            }
            mTextDataId = id;
        }

        void setCallDataId(long id) {// 设定文本数据id
            if (id <= 0) {
                throw new IllegalArgumentException("Call data id should larger than 0");
                // id保证大于0
            }
            mCallDataId = id;
        }

        // 设置电话号码对应的id
        void setCallData(String key, String value) {
            mCallDataValues.put(key, value);
            mNoteDiffValues.put(NoteColumns.LOCAL_MODIFIED, 1);
            // 系统修改时间
            mNoteDiffValues.put(NoteColumns.MODIFIED_DATE, System.currentTimeMillis());
        }

        // 设置文本数据
        void setTextData(String key, String value) {
            mTextDataValues.put(key, value);
            mNoteDiffValues.put(NoteColumns.LOCAL_MODIFIED, 1);
            // 设置修改时间为当前系统时间
            mNoteDiffValues.put(NoteColumns.MODIFIED_DATE, System.currentTimeMillis());
        }

        // 下面函数的作用是将新的数据通过Uri的操作存储到数据库
        Uri pushIntoContentResolver(Context context, long noteId) {
            /**
             * Check for safety
             */
            if (noteId <= 0) {
                throw new IllegalArgumentException("Wrong note id:" + noteId);
            } // 判断数据是否合法

            ArrayList<ContentProviderOperation> operationList = new ArrayList<ContentProviderOperation>();
            ContentProviderOperation.Builder builder = null;// 数据库的操作列表

            if (mTextDataValues.size() > 0) {
                // 把文本数据存入DataColumns
                mTextDataValues.put(DataColumns.NOTE_ID, noteId);// 设定文本数据的属性
                if (mTextDataId == 0) {
                    mTextDataValues.put(DataColumns.MIME_TYPE, TextNote.CONTENT_ITEM_TYPE);
                    // MIME：MIME(MultipurposeInternet Mail Extensions)多用途互联网邮件扩展类型。
                    // 是设定某种扩展名的文件用一种应用程序来打开的方式类型，
                    // 当该扩展名文件被访问的时候，浏览器会自动使用指定应用程序来打开。
                    // 多用于指定一些客户端自定义的文件名，以及一些媒体文件打开方式。
                    Uri uri = context.getContentResolver().insert(Notes.CONTENT_DATA_URI,
                            mTextDataValues);
                    try {// 尝试重新给它设置id
                        setTextDataId(Long.valueOf(uri.getPathSegments().get(1)));
                    } catch (NumberFormatException e) {
                        Log.e(TAG, "Insert new text data fail with noteId" + noteId);
                        // 插入数据失败
                        mTextDataValues.clear();
                        // 把电话号码数据存入DataColumns
                        return null;
                    }
                } else {// 内容提供者的更新操作，因为这个uri对应的数据是已经存在的，所以不需要向上面一样新建，而是更新即可
                    builder = ContentProviderOperation.newUpdate(ContentUris.withAppendedId(
                            Notes.CONTENT_DATA_URI, mTextDataId));// 语句： 将uri和id合并后，更新
                    builder.withValues(mTextDataValues);
                    operationList.add(builder.build());
                }
                mTextDataValues.clear();
            } // 把文本数据存入DataColumns

            if (mCallDataValues.size() > 0) {// 对于电话号码的数据也是和文本数据一样的同步处理
                mCallDataValues.put(DataColumns.NOTE_ID, noteId);// 写入noteID
                if (mCallDataId == 0) {
                    // 将电话号码的id设定为uri提供的id
                    mCallDataValues.put(DataColumns.MIME_TYPE, CallNote.CONTENT_ITEM_TYPE);
                    Uri uri = context.getContentResolver().insert(Notes.CONTENT_DATA_URI,
                            mCallDataValues);
                    try {// 异常处理
                        setCallDataId(Long.valueOf(uri.getPathSegments().get(1)));
                        // 异常处理，返回1
                    } catch (NumberFormatException e) {
                        Log.e(TAG, "Insert new call data fail with noteId" + noteId);
                        // 插入电话号码数据失败
                        mCallDataValues.clear();
                        return null;
                    }
                } else {
                    builder = ContentProviderOperation.newUpdate(ContentUris.withAppendedId(
                            Notes.CONTENT_DATA_URI, mCallDataId));
                    // 内容提供者的更新操作，这个uri对应的数据是已经存在的，因此进行更新即可
                    builder.withValues(mCallDataValues);
                    operationList.add(builder.build());
                }
                mCallDataValues.clear();
            } // 把电话号码数据存入DataColumns

            if (operationList.size() > 0) {
                // 存储过程中如果遇到异常，通过如下进行处理
                try {
                    // Android源码中对通讯录的操作，应用端使用ContentProvider提供的applyBatch，进行批量处理通讯录的联系人入库
                    ContentProviderResult[] results = context.getContentResolver().applyBatch(
                            Notes.AUTHORITY, operationList);
                    return (results == null || results.length == 0 || results[0] == null) ? null
                            : ContentUris.withAppendedId(Notes.CONTENT_NOTE_URI, noteId);
                } catch (RemoteException e) {
                    Log.e(TAG, String.format("%s: %s", e.toString(), e.getMessage()));
                    // 异常日志
                    return null;
                } catch (OperationApplicationException e) {
                    Log.e(TAG, String.format("%s: %s", e.toString(), e.getMessage()));
                    return null;
                }
            } // 存储过程中的异常处理
            return null;
        }
    }
}
